
seconds(n)
  0s + n

body
  foo 15px - 5px
  foo 15px - 5
  
  // 60.8mm
  foo 5cm - 2mm
  foo 10mm + 2in
  
  // 3.54cm
  foo 1cm + 1in
  
  // 3in
  foo 5in - 5.08cm
  
  // 0.8s
  foo 1s - 200ms
  
  // 3500ms
  foo 1500ms + 2s
  
  // 3000Hz
  foo 1000Hz + 2kHz
  
  // 1kHz
  foo 3kHz - (1000Hz * 2)
  
  n = 30%
  foo unit(n, 'px')
  
  foo seconds(1500ms)