
font-families = "Lucida Grande", Arial, sans-serif
font-size = 14px
primary-color = #3a3a3a
width = 800px
pad = 20px

body
  font font-size/1.5 font-families
  background white
  color primary-color

h1
  font-size font-size * (5 - 3)
  color primary-color + #030303

h2
  color primary-color + 20%

#wrapper
  width width

.side-bar
  padding pad
  width (width / 3) - pad
  // 6
  height 3 - - - - 3
  // 7
  height 1 + 2 * 3