
// foo = 'bar'

body
  color red
  // lots of stuff
  // wahoo

  // super cool

  // background green

form
  background white

/*

body
  a
    color blue

*/
// rawr

.abc
  background: // comment here,
    url("img.png") 8px 8px no-repeat,
    rgba(0, 0, 0, .41);

.popup
  -webkit-box-shadow: 1px 0 0 white, /* TODO */ 2px 0 0 black
  -moz-box-shadow: 1px 0 0 white /* TODO */ 2px 0 0 black

.foo
  color: red; /* inline */
  width: 100px;
  /* foo */
  height: 300px;

// comment with ', ", // and :
@media only screen and (max-width: 1230px)
  .test
    foo: bar
