body {
  color: 204;
}
body {
  font: 14px 'Lucida Grande', Arial, sans-serif;
  padding: 5px;
  margin: 5px;
}
a {
  font: 11px Impact, 'Lucida Grande', Arial, sans-serif, serif;
  border: 1px solid #fff;
}