.foo {
  qux: '.foo';
}
.foo .bar {
  foo: '.foo .bar';
}
.foo .bar:hover {
  bar: '.foo .bar:hover';
}
.foo .bar {
  baz: '.foo .bar';
}
.foo,
.bar {
  baz: '.foo,.bar';
}
.foo {
  baz: '.foo';
}
.bar {
  baz: '.bar';
}
broken selector {
  foo: 'broken selector';
}
.foo .foo .bar {
  baz: '.foo .foo .bar';
}
.foo .foo .bar:hover {
  foo: '.foo .foo .bar:hover';
}
.foo {
  baz: '.foo:hover';
  baz: 'li a';
  baz: '.foo:hover, .foo:active';
}
.qux .qux {
  baz: '.qux .qux__qux';
}
.foo,
.bar {
  baz: '.foo:hover,.bar:hover';
  baz: '.foo:hover,.bar:hover';
}
