
// This is a comment
body
  color white
  // You can nest them as you wish
  // and have more comments
  // and more!

/*
We can also use multi-line comments
like this, which are native to CSS
*/

form
  /* color black */
