
image-width(path)
  return image-size(path)[0]

image-height(path)
  return image-size(path)[1]

body
  font add(5px, 10px)
  font sub(5px, 10px)

#jesus
  width image-width('images/jesus.gif')
  height image-height('images/jesus.gif')

#jesus-2
  width image-size('images/jesus.gif')[0]
  height image-size('images/jesus.gif')[1]

#jesus-3
  size = image-size('images/jesus.gif')
  width size[0]
  height size[1]