
body
  background url("http://foo.com/bar.png")

#wrapper
  background url(/images/gopher.jpg)

#footer
  background url(/images/gradient.svg)

.sprite
  // both work due to the additional paths option
  // passed to stylus.url()
  background url(/images/sprite.gif)
  background url(sprite.gif)
