body {
  color: #f00;
}
form {
  background: #fff;
}
/*

body
  a
    color blue

*/
.abc {
  background: url("img.png") 8px 8px no-repeat, rgba(0,0,0,0.41);
}
.popup {
  -webkit-box-shadow: 1px 0 0 #fff, 2px 0 0 #000;
  -moz-box-shadow: 1px 0 0 #fff 2px 0 0 #000;
}
.foo {
  color: #f00; /* inline */
  width: 100px;
/* foo */
  height: 300px;
}
@media only screen and (max-width: 1230px) {
  .test {
    foo: bar;
  }
}
