
border-radius()
  -webkit-border-radius arguments
  -moz-border-radius arguments
  border-radius arguments

form
  border-radius 5px

a.button
  border-radius 10px


support-for-ie ?= true

opacity(n)
  opacity n
  if support-for-ie
    filter unquote('progid:DXImageTransform.Microsoft.Alpha(Opacity=' + round(n * 100) + ')')

#logo
  &:hover
    opacity 0.5
