
size = 14px
font-family = 'Lucida Grande', Arial, sans-serif
$type = solid
blue = #00c

body
  color blue(blue)

body
  font size font-family
  padding pad = 5px
  margin pad

a
  font 11px Impact, font-family, serif
  border 1px $type #fff